"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.StaticGenerator = void 0;
const path_1 = __importDefault(require("path"));
const fs_1 = __importDefault(require("fs"));
const ts = __importStar(require("typescript"));
const esbuild = __importStar(require("esbuild"));
const base_1 = require("@plutolang/base");
const utils_1 = require("./utils");
// The name of the compiled entrypoint
const ENTRYPOINT_FILENAME = "pulumi";
// The name of the compiled compute module for each resource
const COMP_MOD_FILENAME = (resName) => `${resName}`;
class StaticGenerator {
    async generate(opts) {
        const compiledDir = path_1.default.join(opts.outdir, "compiled");
        const pirTsCode = genPirCode(opts.archRef);
        (0, utils_1.writeToFile)(opts.outdir, ENTRYPOINT_FILENAME + ".ts", pirTsCode);
        const pirJsCode = compileTs(pirTsCode);
        (0, utils_1.writeToFile)(compiledDir, ENTRYPOINT_FILENAME + ".js", pirJsCode);
        const cirCodes = genAllCirCode(opts.archRef);
        cirCodes.forEach((cir) => {
            const cirTsPath = COMP_MOD_FILENAME(cir.resource.name) + ".ts";
            (0, utils_1.writeToFile)(opts.outdir, cirTsPath, cir.code);
            bundle(path_1.default.join(opts.outdir, cirTsPath), compiledDir);
        });
        return path_1.default.join(compiledDir, ENTRYPOINT_FILENAME + ".js");
    }
}
exports.StaticGenerator = StaticGenerator;
function bundle(tsPath, outdir) {
    const result = esbuild.buildSync({
        bundle: true,
        minify: false,
        entryPoints: [tsPath],
        platform: "node",
        target: "node18",
        outdir: outdir,
    });
    if (result.errors.length > 0) {
        throw new Error("Failed to bundle: " + result.errors[0].text);
    }
}
function compileTs(code) {
    return ts.transpileModule(code, {
        compilerOptions: { module: ts.ModuleKind.CommonJS },
    }).outputText;
}
function genPirCode(archRef) {
    let iacSource = `// Register all IaC SDK
import { Registry } from "@plutolang/base";

const RUNTIME_TYPE = process.env['RUNTIME_TYPE'];
if (!RUNTIME_TYPE) {
  throw new Error("Missing RUNTIME_TYPE");
}
const ENGINE_TYPE = process.env['ENGINE_TYPE'];
if (!ENGINE_TYPE) {
  throw new Error("Missing ENGINE_TYPE");
}
const reg: Registry = new Registry();

import { register as plutoRegister } from "@plutolang/pluto-infra";
plutoRegister(reg);


let resDefCls;

`;
    // Resource definition, first for BaaS, second for FaaS
    for (let resName in archRef.resources) {
        const res = archRef.getResource(resName);
        if (res.type == "Root" || res.type == "FnResource")
            continue;
        iacSource += res.getImports().join("\n") + "\n";
        iacSource += `resDefCls = reg.getResourceDef(RUNTIME_TYPE, ENGINE_TYPE, ${res.type});
const ${resName} = new resDefCls(${res.getParamString()});\n\n`;
    }
    // Specify the dependency of FaaS on this particular BaaS, because the building image process needs to be performed after exporting Dapr YAML.
    for (let resName in archRef.resources) {
        const res = archRef.getResource(resName);
        if (res.type != "FnResource")
            continue;
        const deps = [];
        for (let relat of archRef.relationships) {
            if (relat.from != res || relat.type != base_1.arch.RelatType.ACCESS)
                continue;
            deps.push(relat.to.name);
        }
        iacSource += `resDefCls = reg.getResourceDef(RUNTIME_TYPE, ENGINE_TYPE, "${res.type}");
const ${resName} = new resDefCls(${res.getParamString()}, {}, { dependsOn: [${deps.join(",")}] });\n\n`;
    }
    // Establish resource dependencies, including triggering and accessing.
    for (let relat of archRef.relationships) {
        if (relat.from.type == "Root")
            continue;
        if (relat.type == base_1.arch.RelatType.CREATE) {
            iacSource += `${relat.from.name}.${relat.operation}(${relat.getParamString()});\n`;
        }
        else if (relat.type == base_1.arch.RelatType.ACCESS) {
            iacSource += `${relat.from.name}.getPermission("${relat.operation}", ${relat.to.name});\n`;
        }
    }
    iacSource += "\n";
    let outputed = false;
    for (let resName in archRef.resources) {
        const res = archRef.getResource(resName);
        if (res.type == "Root")
            continue;
        iacSource += `${resName}.postProcess();\n`;
        // TODO: update the output mechanism
        if (res.type == "Router" && !outputed) {
            outputed = true;
            iacSource += `export const { url } = ${res.name};\n`;
        }
    }
    return iacSource;
}
function genAllCirCode(archRef) {
    let rootRes = archRef.getResource("App");
    const genCirCode = (res) => {
        let cirCode = res.getImports().join("\n") + "\n";
        // Append all direct import statments to generated code
        cirCode += rootRes.getImports().join("\n") + "\n";
        // Find the dependencies of this CIR and build corresponding instances.
        for (let relat of archRef.relationships) {
            if (relat.from != res)
                continue;
            // TODO: verify if the buildClient function exists. If it does not, use the original statement.
            cirCode += relat.to.getImports() + "\n";
            cirCode += `const ${relat.to.name} = ${relat.to.type}.buildClient(${relat.to.getParamString()});\n`;
        }
        // TODO: Assuming there is only one loction now.
        res.locations.forEach((loc) => {
            const usercode = fs_1.default.readFileSync(loc.file, "utf-8");
            const lines = usercode.split("\n");
            const [startLine, startPos] = loc.linenum["start"].split("-").map((n) => Number(n));
            const [endLine, endPos] = loc.linenum["end"].split("-").map((n) => Number(n));
            for (let lineIdx = startLine; lineIdx <= endLine; lineIdx++) {
                const linecode = lines[lineIdx];
                let partcode = "";
                if (lineIdx == startLine) {
                    partcode = `export default ` + linecode.slice(startPos);
                }
                else if (lineIdx == endLine) {
                    partcode = linecode.substring(0, endPos);
                }
                else {
                    partcode = linecode;
                }
                cirCode += partcode + "\n";
            }
        });
        return cirCode;
    };
    const cirs = [];
    for (let resName in archRef.resources) {
        const res = archRef.getResource(resName);
        if (res.type != "FnResource")
            continue;
        cirs.push({ resource: res, code: genCirCode(res) });
    }
    return cirs;
}
//# sourceMappingURL=generator.js.map